#!/bin/sh

# DEBUG=1 to see commands being ran
[ "$DEBUG" ] && set -x

secAdvName="SecurityAdvisory0030"
backupFilePath="/etc/""$secAdvName""/backupProg.tgz"

dnsmasqOld="/etc/""$secAdvName""/dnsmasq-2.59-patched.tgz"

function isProgramRunning {
    # Check if program to be patched is currently running
    service dnsmasq status
    isProgramRunningRetval=$?
}

function backupProgram {
    # Backup program to be patched as a tar file in /tmp
    tar czvf "$backupFilePath" --absolute-names $(rpm -ql dnsmasq)
    # Skip checking retval since FAQ and other files do not get installed and
    # leads to errors (?!)
}

function checkVersion {
    # Check the version of the program and optionally return the patch file to use
    dnsmasqVer=$(rpm -q --queryformat '%{VERSION}' dnsmasq)
    if [ "$dnsmasqVer" == "2.59" ]
    then
        checkVersionRetval="$dnsmasqOld"
    else
        logger -t "$secAdvName" "FAILURE: Did not recognize version of dnsmasq. Please contact TAC team in case of questions."
        exit 1
    fi
}

function patchProgram {
    # Install the correct patch to fix the program, optionally
    # based on the version of the SWI
    tar xvzf "$checkVersionRetval" --absolute-names --overwrite --unlink-first --no-overwrite-dir
    if [ "$?" -ne 0 ]
    then
        logger -t "$secAdvName" "FAIL: Unable to patch program fully."
    fi
}

function restartProgram {
    # Restart the program
    if [[ `systemctl` =~ -\.mount ]]
    then
        # We've had issues with dnsmasq not restarting when the patch extension
        # is installed on systemd images; kill dnsmasq with brute force to make
        # it sure it restarts
        kill -9 `pgrep dnsmasq`
    fi
    service dnsmasq restart
}

function installPatch {
    # Install patch after checking version prereqs to see if the
    # correct patch can be found.
    checkVersion

    # Backup the current version
    backupProgram

    # Check if program is running
    isProgramRunning

    # Install the new patch
    patchProgram

    # Restart the service ( if needed )
    if [ $isProgramRunningRetval -eq 0 ] 
    then
        restartProgram
    fi

    # Log for patch being successful
    logger -t "$secAdvName" "Patch installed successfully"
}

function revertPatch {
    # Uninstall the patch by replacing it with the original files
    tar xvzf "$backupFilePath" --absolute-names --overwrite --unlink-first --no-overwrite-dir
    if [ "$?" -ne 0 ]
    then
        logger -t "$secAdvName" "FAIL: Unable to revert patch fully."
    fi
}

function uninstallPatch {
    # Check if the backup file exists. If it does, restore the previous files
    # then restart the program.
    if [ ! -e "$backupFilePath" ]
    then
        logger -t "$secAdvName" "FAILURE: Failed to find backup file. Please contact TAC team in case of questions."
        exit 1
    fi

    # Check if the program is running
    isProgramRunning

    # Revert patch
    revertPatch

    # Restart the service ( if needed )
    if [ $isProgramRunningRetval -eq 0 ] 
    then
        restartProgram
    fi

    # Log for patch being successful
    logger -t "$secAdvName" "Patch removed successfully"
}

if [ $A4_CHROOT ]
then
    echo "Bailing out early since in dev workspace"
    exit 0
fi

if [ "$1" == "--install" ];
then
    installPatch
elif [ "$1" == "--uninstall" ];
then
    uninstallPatch
else
    unrecogStr="Unrecognized argument to $0 . Please specify --install or --uninstall."
    echo "$unrecogStr" 1>&2
    logger -t "$secAdvName" "$unrecogStr"
fi
